package ir.tasks.twelve.d;

import ir.tasks.twelve.a.Frequency;
import ir.tasks.twelve.a.Utilities;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class PalindromeFrequencyCounter {
	/**
	 * This class should not be instantiated.
	 */
	private PalindromeFrequencyCounter() {}
	
	/**
	 * Takes the input list of words and processes it, returning a list
	 * of {@link Frequency}s.
	 * 
	 * This method expects a list of lowercase alphanumeric strings.
	 * If the input list is null, an empty list is returned.
	 * 
	 * There is one frequency in the output list for every 
	 * unique palindrome found in the original list. The frequency of each palindrome
	 * is equal to the number of times that palindrome occurs in the original list. Smaller palindromes
	 * within larger palindromes should not be separately counted.  Overlapping
	 * palindromes should be counted separately.
	 * 
	 * Palindromes can span sequential words in the input list.
	 * 
	 * The returned list is ordered by decreasing frequency, with tied palindromes sorted
	 * alphabetically. 
	 * 
	 * The original list is not modified.
	 *
	 * In a literary sense, palindromes end on word boundaries.  For our
	 * purposes palindromes can start and end within words.
	 *
	 * Only return palindromes longer than 5 characters.  Spaces do not count
	 * and should not be included in the returned list.
	 * 
	 * Example:
	 * 
	 * Given the input list of strings 
	 * ["do", "geese", "see", "god", "abba", "bat", "tab","xxbap","bat", "tab"]
	 * 
	 * The output list of palindromes should be 
	 * [ "battab:2", "abxxba:1", "dogeeseseegod:1"]
	 *  
	 * @param words A list of words.
	 * @return A list of palindrome frequencies, ordered by decreasing frequency.
	 */
	public static List<Frequency> computePalindromeFrequencies(List<String> words) {
		// TODO Write body!
		// You will likely want to create helper methods / classes to help implement this functionality
		return null;
	}


	/**
	 * Same as computePalindromeFrequencies except it only includes palindromes
	 * that end on word boundaries.
	 * 
	 * Example:
	 * 
	 * Given the input list of strings 
	 * ["do", "geese", "see", "god", "abba", "bat", "tab","xxbap","bat", "tab"]
	 * 
	 * The output list of palindromes should be 
	 * [ "battab:2", "dogeeseseegod:1"]
	 *  
	 * @param words A list of words.
	 * @return A list of palindrome frequencies, ordered by decreasing frequency.
	 */
	public static List<Frequency> computeWordAwarePalindromeFrequencies(List<String> words) {
		// TODO Optional for extra credit!
		return null;
	}
	
	/**
	 * Runs the palindrome finder. The input should be the path to a text file.
	 * 
	 * @param args The first element should contain the path to a text file.
	 */
	public static void main(String[] args) {
		File file = new File(args[0]);
		ArrayList<String> words = Utilities.tokenizeFile(file);
		List<Frequency> frequencies = computePalindromeFrequencies(words);
		Utilities.printFrequencies(frequencies);
	}
}
